@@ -35,13 +35,12 @@
 #include <near/log.h>
 #include <near/types.h>
 #include <near/adapter.h>
-#include <near/target.h>
 #include <near/tag.h>
 #include <near/ndef.h>
 #include <near/tlv.h>
 
-extern int mifare_read_tag(uint32_t adapter_idx, uint32_t target_idx,
-		near_tag_io_cb cb, enum near_target_sub_type tgt_subtype);
+extern int mifare_read(uint32_t adapter_idx, uint32_t target_idx,
+		near_tag_io_cb cb, enum near_tag_sub_type tgt_subtype);
 
 #define CMD_READ         0x30
 #define CMD_WRITE        0xA2
@@ -125,12 +124,15 @@ static int data_recv(uint8_t *resp, int length, void *data)
 	memcpy(nfc_data + current_length, resp + NFC_HEADER_SIZE, length_read);
 
 	if (current_length + length_read == data_length) {
+		GList *records;
+
 		/* TODO parse tag->data for NDEFS, and notify target.c */
 		tag->current_block = 0;
 
 		DBG("Done reading");
 
-		near_tlv_parse(tag->tag, tag->cb);
+		records = near_tlv_parse(nfc_data, data_length);
+		near_tag_add_records(tag->tag, records, tag->cb, 0);
 
 		g_free(tag);
 
@@ -196,8 +198,12 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 		goto out;
 	}
 
-	tag = near_target_add_tag(cookie->adapter_idx, cookie->target_idx,
-				  NULL, TAG_DATA_LENGTH(cc));
+	err = near_tag_add_data(cookie->adapter_idx, cookie->target_idx,
+					NULL, TAG_DATA_LENGTH(cc));
+	if (err < 0)
+		goto out;
+
+	tag = near_tag_get_tag(cookie->adapter_idx, cookie->target_idx);
 	if (tag == NULL) {
 		err = -ENOMEM;
 		goto out;
@@ -213,8 +219,6 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 	t2_tag->cb = cookie->cb;
 	t2_tag->tag = tag;
 
-	near_tag_set_uid(tag, resp + NFC_HEADER_SIZE, 8);
-
 	/* Set the ReadWrite flag */
 	if (TAG_T2_WRITE_FLAG(cc) == TYPE2_NOWRITE_ACCESS)
 		near_tag_set_ro(tag, TRUE);
@@ -260,15 +264,15 @@ static int nfctype2_read_meta(uint32_t adapter_idx, uint32_t target_idx,
 							meta_recv, cookie);
 }
 
-static int nfctype2_read_tag(uint32_t adapter_idx,
+static int nfctype2_read(uint32_t adapter_idx,
 				uint32_t target_idx, near_tag_io_cb cb)
 {
 	int err;
-	enum near_target_sub_type tgt_subtype;
+	enum near_tag_sub_type tgt_subtype;
 
 	DBG("");
 
-	tgt_subtype = near_target_get_subtype(adapter_idx, target_idx);
+	tgt_subtype = near_tag_get_subtype(adapter_idx, target_idx);
 
 	switch (tgt_subtype) {
 	case NEAR_TAG_NFC_T2_MIFARE_ULTRALIGHT:
@@ -278,7 +282,7 @@ static int nfctype2_read_tag(uint32_t adapter_idx,
 	/* Specific Mifare read access */
 	case NEAR_TAG_NFC_T2_MIFARE_CLASSIC_1K:
 	case NEAR_TAG_NFC_T2_MIFARE_CLASSIC_4K:
-		err= mifare_read_tag( adapter_idx, target_idx,
+		err= mifare_read( adapter_idx, target_idx,
 			cb, tgt_subtype);
 		break;
 
@@ -385,19 +389,19 @@ out:
 	return err;
 }
 
-static int nfctype2_write_tag(uint32_t adapter_idx, uint32_t target_idx,
+static int nfctype2_write(uint32_t adapter_idx, uint32_t target_idx,
 				struct near_ndef_message *ndef,
 				near_tag_io_cb cb)
 {
 	struct near_tag *tag;
-	enum near_target_sub_type tgt_subtype;
+	enum near_tag_sub_type tgt_subtype;
 
 	DBG("");
 
 	if (ndef == NULL || cb == NULL)
 		return -EINVAL;
 
-	tag = near_target_get_tag(adapter_idx, target_idx);
+	tag = near_tag_get_tag(adapter_idx, target_idx);
 	if (tag == NULL)
 		return -EINVAL;
 
@@ -406,7 +410,7 @@ static int nfctype2_write_tag(uint32_t adapter_idx, uint32_t target_idx,
 		return -EPERM;
 	}
 
-	tgt_subtype = near_target_get_subtype(adapter_idx, target_idx);
+	tgt_subtype = near_tag_get_subtype(adapter_idx, target_idx);
 
 	if (tgt_subtype != NEAR_TAG_NFC_T2_MIFARE_ULTRALIGHT) {
 		DBG("Unknown Tag Type 2 subtype (%d)", tgt_subtype);
@@ -480,8 +484,8 @@ out:
 static struct near_tag_driver type2_driver = {
 	.type           = NFC_PROTO_MIFARE,
 	.priority       = NEAR_TAG_PRIORITY_DEFAULT,
-	.read_tag       = nfctype2_read_tag,
-	.add_ndef       = nfctype2_write_tag,
+	.read           = nfctype2_read,
+	.write          = nfctype2_write,
 	.check_presence = nfctype2_check_presence,
 };
 
